@@ -2,10 +2,10 @@
  * 
  * cgi.c -
  * 
- * $Id: cgi.c,v 1.61.4.10 2002/06/06 10:48:46 knok Exp $
+ * $Id: cgi.c,v 1.61.4.15 2004/03/20 15:02:51 usu Exp $
  * 
  * Copyright (C) 1997-1999 Satoru Takabayashi All rights reserved.
- * Copyright (C) 2000 Namazu Project All rights reserved.
+ * Copyright (C) 2000-2004 Namazu Project All rights reserved.
  * This is free software with ABSOLUTELY NO WARRANTY.
  * 
  * This program is free software; you can redistribute it and/or modify
@@ -180,12 +180,15 @@ add_cgivar(struct cgivar *ptr, const char *name, char *value)
     tmp->name = malloc(strlen(name) + 1);
     if (tmp->name == NULL) {
 	 printf("add_cgivar_malloc");
+         free(tmp);
 	 return NULL;
     }
 
     tmp->value = malloc(strlen(value) + 1);
     if (tmp->value == NULL) {
 	 printf("add_cgivar_malloc");
+         free(tmp->name);
+         free(tmp);
 	 return NULL;
     }
 
@@ -225,18 +228,15 @@ get_query_string(void)
         if (script_name == NULL) {
             return NULL;
         }
-#if  defined(_WIN32)
     } else if (!(nmz_strprefixcasecmp(getenv("SERVER_SOFTWARE"), "Microsoft-"))
-            || !(nmz_strprefixcasecmp(getenv("SERVER_SOFTWARE"), "AnWeb"))) {
+            || !(nmz_strprefixcasecmp(getenv("SERVER_SOFTWARE"), "AnWeb"))
+            || !(nmz_strprefixcasecmp(getenv("SERVER_SOFTWARE"), "iPlanet-"))) {
 	query_string = "";
 	script_name = getenv("SCRIPT_NAME");
         if (script_name == NULL) {
             return NULL;
         }
     } else {
-#else
-    } else {
-#endif
 	/* Must not be reached here. */
 	assert(0);
     }
@@ -254,6 +254,8 @@ get_cgi_vars(const char *querystring)
 	char name[BUFSIZE];
 	char value[BUFSIZE];
 
+        name[BUFSIZE - 1] = '\0';
+
 	tmp = strchr(querystring, '=');
 	if (tmp == NULL) {
 	    break;
@@ -486,7 +488,7 @@ process_cgi_var_idxname(char *value, struct cgiarg *ca)
     char *pp;
 
     for (pp = value; *pp ;) {
-	char name[BUFSIZE], tmp[BUFSIZE], *x;
+	char name[BUFSIZE] = "", tmp[BUFSIZE] = "", *x;
 
 	if ((x = (char *)strchr(pp, (int)','))) {
 	    *x = '\0';
@@ -501,10 +503,10 @@ process_cgi_var_idxname(char *value, struct cgiarg *ca)
 	validate_idxname(name);
 	remove_ending_slashes(name);
 
-	strncpy(tmp, nmz_get_defaultidx(), BUFSIZE);
-	strncat(tmp, "/", BUFSIZE - strlen(tmp));
-	strncat(tmp, name, BUFSIZE - strlen(tmp));
-	tmp[BUFSIZE -1] = '\0';
+	strncpy(tmp, nmz_get_defaultidx(), BUFSIZE - 1);
+	strncat(tmp, "/", BUFSIZE - strlen(tmp) - 1);
+	strncat(tmp, name, BUFSIZE - strlen(tmp) - 1);
+	tmp[BUFSIZE - 1] = '\0';
 	if (nmz_add_index(tmp) != SUCCESS) {
 	    nmz_warn_printf("invalid idxname: %s", name);
 	}
